/*
    STELE
    Copyright (C) 2006  
	Borca Bogdan <borca_bn@yahoo.com>
	www.bogdi.ro

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "calcule.h"

/*	PO[planet ID][x] where x is one of the following:
	[0]		4878 				diameter of the planet
			/mercurymap.jpg 	texture
	[1]		4.092353			daily motion
	[2]		0.38709893			a_mean_distance_AU
	[3]		0.00000066			a_mean_distance_AU_Cy
	[4]		0.20563069			e_eccentricity_AU
	[5]		0.00002527			e_eccentricity_AU_Cy
	[6]		7.00487				i_inclination_deg_AU inclination (deg)
	[7]		-23.51				i_inclination_deg_AU_Cy
	[8]		48.333899			o_longit_of_asc_node_AU longitude of ascending node (deg)
	[9]		0					o_longit_of_asc_node_AU_Cy
	[10]	77.45645			w_longit_of_perih_deg_AU the position in the orbit where the planet is closest to the Sun
	[11]	573.57				w_longit_of_perih_deg_AU_Cy
	[12]	314.423688			L_mean_longit_deg_AU mean longitude (deg), the position of the planet in the orbit
	[13]	0					number_of_moons
	[14]	0.01				axial_tilt	
	[15]	0					years, in Earth years, year
	[16]	0					color of the planet
	[17]	6.138505			rotation rate (in degrees/Earth day)
*/

double 
// day, centuries
	D_day_number, 
	Cy_centuries,
// positional data based on time/date
	a_mean_distance, 
	e_eccentricity, 
	i_inclination, 
	O_long_asc_node,
	w_long_perihelion,		// the position in the orbit where the planet is closest to the Sun
	L_mean_long,			// (degress), the position of the planet in orbit
// Later calculations based on critical data, calculated quantities
	M_mean_anomaly,
	v_true_anomaly,
	R_heli_redius,
	E,
	Days;

CAAGalileanMoonsDetails GalileanDetails;
CAASaturnMoonsDetails saturnMoons;

unsigned short IDP;			// ID of the planet

GLvoid c_time()								// Step 1 : Get the time/date
{
	GetSystemTime(&systemtime);
	time_date = time(NULL);					// take the number of seconds from 1970                
}

GLvoid c_day_number()						// Step 2 : Calculate the days, D_day_number, Cy_centuries
{
	mili =  systemtime.wSecond + systemtime.wMilliseconds / 1000.0L;
	Days = (time_date + systemtime.wMilliseconds / 1000.0L) / 3600.0L / 24.0L - 10092.0L;	
	double h = systemtime.wHour + systemtime.wMinute / 60.0 + (systemtime.wSecond + (systemtime.wMilliseconds/1000.0)) / 3600.0;
	D_day_number = 367 * (systemtime.wYear) - 7 * (systemtime.wYear + (systemtime.wMonth + 9) / 12) / 4 + 
	       275 * systemtime.wMonth/9 + systemtime.wDay - 730531.5 + h / 24;
	Cy_centuries = D_day_number / 36525.0L;
}

GLvoid c_quantities()						// Step 3
{
	a_mean_distance = PO[IDP][2] + PO[IDP][3] * Cy_centuries;
	e_eccentricity = PO[IDP][4] + PO[IDP][5] * Cy_centuries;
	i_inclination = (PO[IDP][6] + PO[IDP][7] * Cy_centuries / 3600.0L) * Radian;
	O_long_asc_node = (PO[IDP][8] + PO[IDP][9] * Cy_centuries / 3600.0L) * Radian;
	w_long_perihelion = (PO[IDP][10] + PO[IDP][11] * Cy_centuries / 3600.0L) * Radian;
	L_mean_long = (PO[IDP][12] + 0 * Cy_centuries / 3600.0L) * Radian;
}

GLvoid c_meananomaly()						// Step 4 : Calculate mean anomaly
{
//	[1] daily_motion; 
	M_mean_anomaly =  PO[IDP][1] * (Days)* Radian + (L_mean_long) - (w_long_perihelion);
}

GLvoid c_trueanomaly()						// Step 5 : Calculate true anomaly
{
	double d, delta;
	E = M_mean_anomaly;
	d = E - e_eccentricity * sin(E) - M_mean_anomaly;
	while (fabs(d) > 1.0E-2) {
       delta = d / (1.0L - e_eccentricity * cos(E));
	   E = E - delta;
	   d = E - e_eccentricity * sin(E) - M_mean_anomaly;
	}
    // convert eccentric anomaly to true anomaly
    v_true_anomaly = 2.0L * atan(sqrt((1.0L + e_eccentricity)/
       (1.0L - e_eccentricity)) * tan(E / 2.0L));
}

GLvoid c_heliocentricradius()				// Step 6 : Calculate the heliocentric radius
{
   R_heli_redius = (a_mean_distance) * (1.0L - (e_eccentricity) * cos(E));
}

// Step 7 : Calculate x, y, z of the planet
GLvoid c_position()
{
//	[18] x_coord planet coordinates X Y Z
//	[19] y_coord
//	[20] z_coord
	double R_AU = R_heli_redius * AU;
	double vwo = v_true_anomaly + w_long_perihelion - O_long_asc_node; 
	PO[IDP][18] =  R_AU * 
		(cos(O_long_asc_node) * cos(vwo) -
		 sin(O_long_asc_node) * sin(vwo) *  cos(i_inclination));
	PO[IDP][19] = R_AU * 
		(sin(O_long_asc_node) * cos(vwo) +
		 cos(O_long_asc_node) * sin(vwo) *  cos(i_inclination));
	PO[IDP][20] = R_AU *
		(sin(vwo) * sin(i_inclination));
}

GLvoid c_rotation_degrees()
{
//	[22] rotation_degrees, rotation of the planet in degrees
//  this will make day, night rotation
//	if 12 PM your Earth should be 
//	with your location pointed to Sun
	double i = atan2(PO[IDP][18], PO[IDP][19]) * 180 / PI;
	if (i < 0) i = 360 + i;
	double e = PO[IDP][17] * Days / PO[IDP][17];
	PO[IDP][22] = (e - floor(e)) * 360 - i + 155.0;
}

// Step 8 : Calculate the distance from the sun
GLvoid c_distance_km_from_the_sun()
{
//	[21] distance to the sun
	PO[IDP][21] = dd2p(
		(PO[IDP][18]),
		(PO[IDP][19]),
		(PO[IDP][20]), 0.0f, 0.0f, 0.0f);//
}

// I will / the values to 100.000 'cause I can't draw a 150.000.000 km
// but 1.500 seems ok, this means I will / 100.000 also the diameter of the planet
// and all other drawing values
// Ex: Earth.Diameter = 12.756 => I will draw a sphere with a 0,12756 diameter
GLvoid c_km_translate()
{
	PO[IDP][18] /= 100000.0L;
	PO[IDP][19] /= 100000.0L;
	PO[IDP][20] /= 100000.0L;
}

GLvoid c_xyz(unsigned short planet_ID)
{
	IDP = planet_ID;	// IDP is the ID of the planet
	c_time();
	c_day_number();
	c_quantities();
	c_meananomaly();
	c_trueanomaly();
	c_heliocentricradius();
	c_position();
	c_rotation_degrees();
	c_distance_km_from_the_sun();
	c_km_translate();
}

GLvoid c_Earth_Moon(double *x, double *y, double *z)
{
	double MoonLong = CAAMoon::EclipticLongitude(Julian);
	double MoonLat = CAAMoon::EclipticLatitude(Julian);
	double MoonRadius = CAAMoon::RadiusVector(Julian) / 100000.0L;
	double LAT = MoonLat * PI/180;
	double LON = MoonLong * PI/180;
	*x = -MoonRadius * cos(LAT) * cos(LON);
	*y =  MoonRadius * sin(LAT);
	*z =  MoonRadius * cos(LAT) * sin(LON);	
}

GLvoid c_Galilean_details(double *x, double *y, double *z, short i)
{
	switch (i)
	{
		case 0 : 
			GalileanDetails = CAAGalileanMoons::Calculate(Julian);//2448972.50068);
			*x = GalileanDetails.Satellite1.ApparentRectangularCoordinates.X ; *y = GalileanDetails.Satellite1.ApparentRectangularCoordinates.Y; *z = GalileanDetails.Satellite1.ApparentRectangularCoordinates.Z; break;
		case 1 : *x = GalileanDetails.Satellite2.ApparentRectangularCoordinates.X ; *y = GalileanDetails.Satellite2.ApparentRectangularCoordinates.Y; *z = GalileanDetails.Satellite2.ApparentRectangularCoordinates.Z; break;
		case 2 : *x = GalileanDetails.Satellite3.ApparentRectangularCoordinates.X ; *y = GalileanDetails.Satellite3.ApparentRectangularCoordinates.Y; *z = GalileanDetails.Satellite3.ApparentRectangularCoordinates.Z; break;
		case 3 : *x = GalileanDetails.Satellite4.ApparentRectangularCoordinates.X ; *y = GalileanDetails.Satellite4.ApparentRectangularCoordinates.Y; *z = GalileanDetails.Satellite4.ApparentRectangularCoordinates.Z; break;
	}
}

GLvoid c_Saturn_moons(double *x, double *y, double *z, short i)
{
	switch (i)
	{
		case 0 : 
			saturnMoons = CAASaturnMoons::Calculate(Julian);
			*x = saturnMoons.Satellite1.ApparentRectangularCoordinates.X; *y = saturnMoons.Satellite1.ApparentRectangularCoordinates.Y; *z = saturnMoons.Satellite1.ApparentRectangularCoordinates.Z; break;
		case 1 : *x = saturnMoons.Satellite2.ApparentRectangularCoordinates.X; *y = saturnMoons.Satellite2.ApparentRectangularCoordinates.Y; *z = saturnMoons.Satellite2.ApparentRectangularCoordinates.Z; break;
		case 2 : *x = saturnMoons.Satellite3.ApparentRectangularCoordinates.X; *y = saturnMoons.Satellite3.ApparentRectangularCoordinates.Y; *z = saturnMoons.Satellite3.ApparentRectangularCoordinates.Z; break;
		case 3 : *x = saturnMoons.Satellite4.ApparentRectangularCoordinates.X; *y = saturnMoons.Satellite4.ApparentRectangularCoordinates.Y; *z = saturnMoons.Satellite4.ApparentRectangularCoordinates.Z; break;
		case 4 : *x = saturnMoons.Satellite5.ApparentRectangularCoordinates.X; *y = saturnMoons.Satellite5.ApparentRectangularCoordinates.Y; *z = saturnMoons.Satellite5.ApparentRectangularCoordinates.Z; break;
		case 5 : *x = saturnMoons.Satellite6.ApparentRectangularCoordinates.X; *y = saturnMoons.Satellite6.ApparentRectangularCoordinates.Y; *z = saturnMoons.Satellite6.ApparentRectangularCoordinates.Z; break;
		case 6 : *x = saturnMoons.Satellite7.ApparentRectangularCoordinates.X; *y = saturnMoons.Satellite7.ApparentRectangularCoordinates.Y; *z = saturnMoons.Satellite7.ApparentRectangularCoordinates.Z; break;
		case 7 : *x = saturnMoons.Satellite8.ApparentRectangularCoordinates.X; *y = saturnMoons.Satellite8.ApparentRectangularCoordinates.Y; *z = saturnMoons.Satellite8.ApparentRectangularCoordinates.Z; break;
	}
}
