/*
    STELE
    Copyright (C) 2006  
	Borca Bogdan <borca_bn@yahoo.com>
	www.bogdi.ro

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
	for loading textures I use GLuint list[10][20] declared
	in init.cpp
	using list[][] have the following technology:
	list[][] kipps the IDs for the generated lists from glGenlist
	with glGenlist I generate a sphere with a texture attached to it
	list[][] will contain the IDs returned by glGenList
	So, the list usage is as follow:
		list[k][0] planet's texture
		list[k][1] planet's atmosphere
		list[k][2->9] planet moons, this makes a maximum of 8 moons
		k is the planet
		k goes from 0 to 9 so maximum 10 planets I can put into the scene
	Now.. Mercury,Venus don't have any atmosphere and moons so I'll use 
	list[0 or 1][2->9] for something else:
	list[0][1] is for a generated sphere with Mercury map on it
	list[0][9] is for a generated sphere with Sun map on it
	list[0][8] is for a generated texture of Saturn rings
	list[0][7] is for a generated texture of Uranus rings
	list[0][6] is for a generated texture of Sun fire
	list[0][5] is for a generated texture of Sun flare
	list[0][4] texture for left_planel
	list[0][3] texture for right_panel
*/

#include"file.h"

FILE *CFG;

double v = 0; 
char buffer[300], moon_name[10]; 
short pdata = 0, mdata = 0, take = 0,  k = -1, h = -1, mID = 0;
bool atmosphere = false, satellite = false;

// k for planets, from 0 to the number of planets
// h for moons, from 0 to the number of moons for every planet

void rem_comment(char * buffer)
{	
	// take out comments
	if (strstr(buffer, ";")==NULL) return;
	int i = 0;
	while (buffer[i++] != ';')
	{}
	buffer[i - 1] = '\0';
}

void ltrim(char * buffer)
{
	// take out left spaces
	int i = 0;
	while ((buffer[i] == 9) || (buffer[i] == ' ') || (buffer[i] == 10) || (buffer[i] == '.'))
	{ i++; }
	strcpy(buffer, buffer + i);
}

void rtrim(char * buffer)
{
	// take out right spaces
	int i = strlen(buffer) - 1;
	while ((buffer[i] == 9) || (buffer[i] == ' ') || (buffer[i] == 10))
	{ i--; }
	buffer[i + 1] = '\0';
}

void change_description(char * d)
{
	char description1[1400];
	int i = 1, j = 0, k = 70, sp = 0;

	memset(description1, '\0', 1400);
	strcpy(description1, d);
	memset(d, '\0', 1400);

	while (i < strlen(description1))
	{
		d[j++] = description1[i];
		if ((description1[i] == ' ')||(description1[i]=='@')) sp = i;
		if ((i == k)||(description1[i]=='@')) { 
			j = sp - 1;
			d[j] = '^';
			k = sp + 70;
			i = sp;
			j++;
		}
		i++;
	}
}

bool init_file()
{
	if((CFG = fopen("Conf/Stele.cfg", "r")) == NULL) 
	{
		MessageBox(hWnd, "Error loading configuration file.\nConf/Stele.cfg", "ERROR", MB_OK);
		return false;
		// return true if file is read valid, false if not
	}

	memset(moon_name, '\0', 10);
	return true;
}

bool load_config_file()
{
		fnt_PrintText_lt_off_color(true, listBase[0], true, true, -0.4, -0.04,  true, 128128128 /* the color */, "Loading %s (%s)", planet[k].name, moon_name);			
READ_NEXT:
		if (feof(CFG)) goto DONE;	// while not end of file
		memset(buffer, '\0', 300);	// set buffer to null
		fgets(buffer, 300, CFG);	// take a buffer of 300 chars from Conf/Stele.cfg

		rem_comment(&buffer[0]);	// remove  any comments
		ltrim(&buffer[0]);			// remove left spaces
		rtrim(&buffer[0]);			// remove right spaces

		if (
			(strcmp(buffer,"")==0)||
			(buffer[0] == '{')||
			(buffer[0] == '}')
			) goto READ_NEXT;		// if buffer is null skipp any tests
		
		if (buffer[0]=='@')							// test if buffer is a description of a planet or satellite
		{ take = satellite?7:1; goto TAKE; }
		if ((strstr(buffer, "planet"))!=NULL)		// test if buffer is a name of a planet
		{ take = 0; atmosphere = false; satellite = false; goto TAKE; }	
		if (buffer[0]=='/')							// test if buffer is a texture, can be a planet atmosphere or satellite texture
		{ take = atmosphere?4:(satellite?9:3); goto TAKE; }
		if ((strstr(buffer, "atmosphere"))!=NULL)	// test if arrived in atmosphere zone 
		{ atmosphere = true; goto READ_NEXT; }
		if ((strstr(buffer, "satellite"))!=NULL)	// test if buffer is a name of a satellite 
		{ take = 6; satellite = true; atmosphere = false; goto TAKE; }		
		if (atmosphere)								// take data from an atmosphere zone 
		{ take = 5; goto TAKE;}

		take = satellite?8:2;						// if satellite then take data for satellite else for planet
TAKE:;
		switch (take)
		{
			case 0:	// search for planet
				k++;
				strcpy(planet[k].name, buffer + 7);
				memset(moon_name, '\0', 10);
				pdata = 0;
				mID = 0;
				break;
			case 1:	// search for a planet description
				strcat(planet[k].description, buffer);
				break;
			case 2:	// search for data
				v = (double)atof(buffer);
				PO[k][pdata++] = v;
				break;
			case 3:	// load planet texture
				list[k][0] = generate_sphere_list(PO[k][0]/100000.0L, true, false, true, JPEG_Texture(buffer + 1));
				break;
			case 4:	// load atmosphere texture
				list[k][1] = generate_sphere_list(101 * (PO[k][0]/100000.0L)/100.0L, true, false, false, JPEG_Texture(buffer + 1));
				break;
			case 5:	// take transparency for the atmosphere
				planet[k].transparency = atol(buffer);
				break;
			case 6:	// take satellite name
				h++;
				strcpy(moon[h].name, buffer + 10);
				strcpy(moon_name, moon[h].name);
				planet[k].moonIDs[mID++] = h;
				mdata = 0;
				break;
			case 7: // take satellite's description
				strcat(moon[h].description, buffer);
				break;
			case 8:	// load satellite's data
				v = (double)atof(buffer);
				MO[h][mdata++] = v;
				break;
			case 9:	// load satellite's texture
					// list[k][0] is reserved for planet texture
					// list[k][1] is reserved for atmosphere texture
				MO[h][0] = MO[h][0] / 100000.0L;
					// MO[h][0] it's the diameter of the satellite
				list[k][2 + mID - 1] = generate_sphere_list(MO[h][0], true, false, true, JPEG_Texture(buffer + 1));
				break;
		}
		return false;
DONE:;
	fclose(CFG);
	planets = k;
	fnt_PrintText_lt_off_color(true, listBase[0], true, true, -0.4, -0.04,  true, 128128128 /* the color */, "Initialize planets position");			
	return true;
}

void load_pxyz()
{	// calculate xyz for all planets

	for (short i = 0; i <= planets; i++)
	{	
		c_xyz(i); 
		camera_planet_distance = dd2p(0, 0, 0, PO[i][18], PO[i][20], PO[i][19]);				
		w_get_closest_planets(planet[i].name, PO[i][16], iP, planets, camera_planet_distance);
		change_description(planet[i].description);
	}
	fnt_PrintText_lt_off_color(true, listBase[0], true, true, -0.4, -0.04,  true, 128128128 /* the color */, "Loading Saturn/Uranus rings");			
}