/*
    STELE
    Copyright (C) 2006  
	Borca Bogdan <borca_bn@yahoo.com>
	www.bogdi.ro

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include"general.h"

void set_light_texture_off(bool lt)
{
   switch (lt)
   {   case true:
       {glDisable(GL_LIGHTING);glDisable(GL_TEXTURE_2D);break;}
       case false:
       {glEnable(GL_LIGHTING);glEnable(GL_TEXTURE_2D); break;}
   }
}

GLuint generate_sphere_list(double radius, 
			bool light_on, bool transparent, bool cull_face, GLuint _texture)
{
	GLuint newlist = glGenLists(1);
	glNewList(newlist, GL_COMPILE);
		glBindTexture(GL_TEXTURE_2D, _texture);
		gluSphere(qsphere, radius , 64, 64);
		// / 2.0 I will draw the spheres with diameter twice as large
	glEndList();
	return newlist;
}

Vector3 GetOGLPos(int x, int y)
{

	GLint viewport[4];
	GLdouble modelview[16];
	GLdouble projection[16];
	GLfloat winX, winY, winZ;
	GLdouble posX, posY, posZ;
	Vector3 P;

	glGetDoublev( GL_MODELVIEW_MATRIX, modelview );
	glGetDoublev( GL_PROJECTION_MATRIX, projection );
	glGetIntegerv( GL_VIEWPORT, viewport );

	winX = (float)x;
	winY = (float)viewport[3] - (float)y;
	glReadPixels( x, int(winY), 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &winZ );

	gluUnProject( winX, winY, winZ, modelview, projection, viewport, &posX, &posY, &posZ);

	P.x = posX;
	P.y = posY;
	P.z = posZ;
	return P;
}

void set_material_color(GLfloat mat_color[], 
	GLfloat r, GLfloat g, GLfloat b, GLfloat to_what)
{
	Vector4(mat_color, r, g, b, 1.0);
	glColor4fv(mat_color);
	glColorMaterial(GL_FRONT, to_what);
}

void set_color(long colorid, GLfloat blend, 
			   unsigned short to_what)
{
/*
id	color		R	|G	|B
1	white		255	|255|255
2	red			255	|0	|0
3	green		0	|255|0
4	blue		0	|0	|255
5	black		0	|0	|0
6	yellow		255	|255|0
7	cyan		0	|255|255
8	violet		255	|0	|255
9	gray		128	|128|128
0	orange		240	|145|30	
11	light blue1	64	|128|192	Earth Like
12	light blue2	52	|160|214	Earth Sky Like
13	dark yellow 170 |156|96
14	dark orange	245	|100|25
15	dark cyan	17	|205|249
16	dark violet 148 |100|165
17	dark blue	37	|71	|179
18	light orange255 |203|11
19  light red	232 |89 |38
20  redish		160 |89 |38
21	gray
*/
	GLfloat color[4];
	switch (colorid)
	{
	// case RGB: c[0]=R c[1]=G c[2]=B
	/*1 */	case 255255255: color[0] = 1; color[1] = 1; color[2] = 1; break;
	/*2 */	case 25500: color[0] = 1; color[1] = 0; color[2] = 0; break;
	/*3 */	case 2550: color[0] = 0; color[1] = 1; color[2] = 0; break;
	/*4 */	case 255 : color[0] = 0; color[1] = 0; color[2] = 1; break;
	/*5 */	case 0 : color[0] = 0; color[1] = 0; color[2] = 0; break;
	/*6 */	case 2552550 : color[0] = 1; color[1] = 1; color[2] = 0; break;
	/*7 */	case 255255 : color[0] = 0; color[1] = 1; color[2] = 1; break;
	/*8 */	case 2550255 : color[0] = 1; color[1] = 0; color[2] = 1; break;
	/*9 */	case 128128128 : color[0] = 0.5; color[1] = 0.5; color[2] = 0.5; break;
	/*0 */	case 24014530 : color[0] = 0.94; color[1] = 0.56; color[2] = 0.11; break;
	/*11*/	case 64128192 : color[0] = 0.25; color[1] = 0.5; color[2] = 0.75; break;
	/*12*/	case 52160214 : color[0] = 0.20; color[1] = 0.62; color[2] = 0.83; break;
	/*13*/	case 17015696 : color[0] = 0.66; color[1] = 0.61; color[2] = 0.37; break;
	/*14*/	case 24510025 : color[0] = 0.96; color[1] = 0.39; color[2] = 0.09; break;
	/*15*/	case 17205249 : color[0] = 0.06; color[1] = 0.80; color[2] = 0.97; break;
	/*16*/	case 148100165 : color[0] = 0.58; color[1] = 0.39; color[2] = 0.64; break;
	/*17*/	case 3771179 : color[0] = 0.14; color[1] = 0.27; color[2] = 0.70; break;
	/*18*/	case 25520311 : color[0] = 1; color[1] = 0.79; color[2] = 0.04; break;
	/*19*/	case 1608938 : color[0] = 0.60; color[1] = 0.44; color[2] = 0.14; break;
	/*20*/	case 2326638 : color[0] = 0.80; color[1] = 0.34; color[2] = 0.14; break;
	/*21*/	case 101010 : color[0] = 0.5; color[1] = 0.5; color[2] = 0.5; break;
	/*22*/	case 101011 : color[0] = 0.23; color[1] = 0.33; color[2] = 0.33; break;
	default: /*white*/
		color[0] = 1; color[1] = 1; color[2] = 1; break;
	}
	color[3] = blend;
	switch (to_what)
	{
		case 0 : glColor4fv(color); break;
		case 1 : glMaterialfv(GL_FRONT, GL_AMBIENT, color); break;
		case 2 : glMaterialfv(GL_FRONT, GL_SPECULAR, color); break;
		case 3 : glMaterialfv(GL_FRONT, GL_DIFFUSE, color); break;
		case 4 : glLightfv(GL_LIGHT0, GL_AMBIENT, color); break;
		case 5 : glLightfv(GL_LIGHT0, GL_SPECULAR, color); break;
		case 6 : glLightfv(GL_LIGHT0, GL_DIFFUSE, color); break;
	default: /*glColor3fv*/
		glColor3fv(color);
	}
}

void fadeout(bool *_fade, float fast, 
	SF3dVector *start, SF3dVector end, SF3dVector toxyz, short minus)
{
	// translate something from START(x, y, z) to END(x, y, z) on (x, y, z)
	// toxyz = (0/1, 0/1, 0/1) specify where fade shoud be done on x y or z
	// start is start point from where the translation is started
	// end is end point where translation ends
	// fast is how fast translation is made
	if (!*_fade) 
	{
		glTranslated(start->x, start->y, start->z);
		return;
	}
	if (toxyz.x==1) start->x = start->x + minus*fast;	// if translation is on x then sub start.x
	if (toxyz.y==1) start->y = start->y + minus*fast;
	if (toxyz.z==1) start->z = start->z + minus*fast;
	glTranslated(start->x, start->y, start->z);
	// return true while fade operation is executed
	if (minus>0)
	{
	*_fade = ((toxyz.x==1)&&(start->x<end.x))|| /* false if toxyz!=1 */
		((toxyz.y==1)&&(start->y<end.y))||	
		((toxyz.z==1)&&(start->z<end.z));
	}
	else
	{
	*_fade = ((toxyz.x==1)&&(start->x>end.x))|| /* false if toxyz!=1 */
		((toxyz.y==1)&&(start->y>end.y))||	
		((toxyz.z==1)&&(start->z>end.z));
	}
}

bool mouse_over_quad_area(GLfloat x_m, GLfloat y_m,
   GLfloat x1, GLfloat x2, GLfloat y1, GLfloat y2)
{
   return (x_m > x1) && (x_m < x2) && (y_m > y1) && (y_m < y2);
}

void dr_line(GLfloat x1, GLfloat y1, GLfloat z1,
     GLfloat x2, GLfloat y2, GLfloat z2)
{
   glVertex3f(x1, y1, z1);
   glVertex3f(x2, y2, z2);
}

void dr_ellipse(bool draw, 
       float radius_a, float radius_b, float inclination, 
       GLfloat color[4])
{
   if (!draw) return;
   //glPushMatrix();
   // Save Matrix 
   set_light_texture_off(true);
   //glTranslatef(radius_b - (radius_b + radius_a)/2, 0.0f, 0.0f);
   // Ma translatez la dreapta sau la stanga pentru a desena
   // orbita caci se stie ca soarele nu este pentru nici o planeta
   // centrul orbitei
   //glColor4fv(color);
   //glRotatef(90.0 + inclination, 1.0, 0.0, 0.0);
   glBegin( GL_LINE_LOOP );
    for (int i = 0; i <= 512; i++)
    {glVertex2f((radius_a * cos (PI / 256.0 * i)), 
               (radius_b * sin (PI / 256.0 * i)));
    }
   glEnd();
   //set_light_texture_off(false);   
   glPopMatrix();
   // Restore matrix
}

void dr_quad(short texture, GLuint GL_what, float x1, float y1, float x2, float y2, float z)
{
	if (texture >= 0) glBindTexture(GL_TEXTURE_2D, texture);
	glBegin(GL_what);
		glTexCoord2f(0.0f ,0.0f); glVertex3f(x1, y1, z); 
		glTexCoord2f(0.0f ,1.0f); glVertex3f(x1, y2, z);
		glTexCoord2f(1.0f ,1.0f); glVertex3f(x2, y2, z); 
		glTexCoord2f(1.0f ,0.0f); glVertex3f(x2, y1, z);
	glEnd();  
} 

void set_2D_for_draw(float near_, float far_)
{
   glPushMatrix();
   // Save the matrix
   glLoadIdentity();
   // Reset The Projection Matrix
   // You use the glLoadIdentity() command to clear the currently modifiable matrix for future
   // transformation commands, since these commands modify the current matrix
   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
   glLoadIdentity();
   glOrtho(0, wwidth, wheight, 0, near_, far_);
   // Creates a matrix for projecting two-dimensional coordinates onto the screen and multiplies the
   // current projection matrix by it. The clipping region is a rectangle with the lower-left corner at
   // (left, bottom) and the upper-right corner at (right, top).
}

void unset_2D_for_draw()
{
   glPopMatrix();
   glMatrixMode(GL_MODELVIEW);
   glPopMatrix();
}

void int2str(int int_, char * str_)
// Return a str from an int
{
   int rest = 0; int i = 0; 
   char rstr[10]="";char aux[10]="";
   while (int_ !=0)
   {rest = int_ % 10;
    if (rest < 0) {rest = 0 - rest;};
    rstr[i] = (char)(rest + 48);
    i++;
    int_ = int_ / 10;
   }
   rstr[i] = '\0'; 
   for(int j=0; j<i; j++) {aux[j] = rstr[i-1-j];}
   aux[i] = '\0';
   strcpy(str_, aux);
   return;
}

double mod2pi(double x)
// Return an angle in the range 0 to 2pi radians
{
    double b = x / (2 * PI);
    double a = (2 * PI)*(b - (int)b);  
    if (a < 0) {a = (2 * PI) + a;}
    return a;
}

double dd2p(double x1, double y1, double z1,
       double x2, double y2, double z2)
{
   return sqrt((x1-x2) * (x1-x2) +
          (y1-y2) * (y1-y2) + (z1-z2) * (z1-z2));
}

short charpos(char sir[], char c)
{
	short i = 0;
	while ((sir[i] != '\0') && (sir[i] != c))
	{
		i++;
	}
	return i;
}