/*
    STELE
    Copyright (C) 2006  
	Borca Bogdan <borca_bn@yahoo.com>
	www.bogdi.ro
	
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "scena.h"

#pragma warning(disable : 4996)
// The compiler encountered a function that was marked with deprecated. 
// The function may no longer be supported in a future release. 
// You can turn this warning off with the warning pragma.

double moon_rot, load_time, hours, atm_rot, rot;	

void load_splash_screen(GLuint texture)
{
	list[0][9] = glGenLists(1);
	// generate list for splashscreen
	glNewList(list[0][9], GL_COMPILE);
		glDisable(GL_CULL_FACE);
		glTranslatef(0.07, 0, -0.5);
		glColor4f(0.8f, 0.8f, 0.8f, 1.0f);
		dr_quad(texture, GL_QUADS, -0.2, -0.2, 0.2, 0.2, 0);
		glDisable(GL_TEXTURE_2D);
		glColor4f(1.0f, 0.8f, 0.04f, 1.0f);
		fnt_PrintText_lt_off_color(true, listBase[0], false, false, -0.4f, 0.0f, false, 0 /* the color */, "Loading STELE...");
		glEnable(GL_CULL_FACE);
	glEndList();
}

void display_splash_screen()
{
	// display splash screen
	glCallList(list[0][9]);
}

void unload_splash_screen()
{
	glDeleteLists(list[0][9], 1);
	fnt_PrintText_lt_off_color(true, listBase[0], true, true, -0.4, -0.04,  true, 128128128 /* the color */, "Loading the Sun^Starting STELE");			
}

void init_quadric()
{
	// setting quadric properties
	gluQuadricTexture(qsphere, GL_TRUE);
}

void init_var()
{
	near_planet = moon_rot = atm_rot = hours = rot = iP = 0;	
	Vector4(light_position, 0.0, 0.0, 0.0, 1.0);
	// init default options
	bool_options[0] = true;		// I key: sun-planet distance
	bool_options[1] = true;		// G key: grid
	bool_options[2] = false;	// V key: orthographic/perspective projection, default is perspective projection
	bool_options[3] = false;	// 2 key: lock to planet
	bool_options[4] = true;		// Space key: show planet description
}

void init_material()
{
	// material
	set_material_color(mat_ambient, 1.0, 1.0, 1.0, GL_AMBIENT);
	set_material_color(mat_specular, 0.0, 0.0, 0.0, GL_SPECULAR);
	set_material_color(mat_diffuse, 1.0, 1.0, 1.0, GL_DIFFUSE);
	set_material_color(mat_emission, 0.0, 0.0, 0.0, GL_EMISSION);
}

void init_light()
{
	// light
	GLfloat	light_specular[4];	// laser beam has 100% reflection, 
	GLfloat	light_diffuse[4];
	GLfloat	light_ambient[4];
	GLfloat	light_emission[4];
	
	Vector4(light_specular, 0.4, 0.4, 0.4, 1.0);
	Vector4(light_diffuse, 1.0, 1.0, 1.0, 1.0);
	Vector4(light_ambient, 0.0, 0.0, 0.0, 1.0);
	Vector4(light_emission, 1.0, 1.0, 1.0, 1.0);

	glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);	
	glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
	glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
	glLightfv(GL_LIGHT0, GL_EMISSION, light_emission);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
}

void init_gl()
{
    glClearDepth(1.0);			// enables Clearing Of The Depth Buffer
	glDepthFunc(GL_LEQUAL);		// the Type Of Depth Testing To Do
	glEnable(GL_CULL_FACE);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_COLOR_MATERIAL);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
	glDisable(GL_LIGHTING); 
	glDisable(GL_LIGHT0);
	glEnable(GL_DEPTH_TEST);
}

void scn_init()
{
	init_var();
	init_light();	
	init_material();
	init_quadric();
	init_gl();
}

void set_light()
{
	// set the light to 0, 0, 0, is where the sun is
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
}

void unload_quadric(void)
{
	// remove the quadric
	gluDeleteQuadric(qsphere);
}

bool pxyz_range()
{
	// return true if planet is in 3 closest planets to camera
	short iPcp = 0;	bool among3planets = false;	
	while ((iPcp < 3) && !(among3planets=(cp[iPcp].ID==iP)))
	{ iPcp++; }
	return among3planets;
}

void draw_system(SF3dVector camera)
{
	glDisable(GL_TEXTURE_2D);
	for (iP=0; iP <=planets;iP++)
	{

		bool among3planets = pxyz_range();
		if (among3planets)  { c_xyz(iP); }	// calculate planet position if among 3 closest
		
		camera_planet_distance = dd2p(		/* get the distance between planet and camera */
			camera.x, camera.y, camera.z, PO[iP][18], PO[iP][19], PO[iP][20]);
			
		w_get_closest_planets(				/* draw only spheres for the three closest planets */
				planet[iP].name, PO[iP][16], iP, planets, camera_planet_distance);

		glPushMatrix();						// saving SCENE 0, begin to draw in SCENE 1
		{
			glTranslated(PO[iP][18], PO[iP][19], PO[iP][20]);
			w_planet_name();
			if ((!among3planets)||(150 < camera_planet_distance)) goto SKIPP_ALL;
			if (near_planet!=iP + 1) near_planet = iP + 1;

			glPushMatrix();
			// saving SCENE 1, begin to draw in SCENE 2
			{
				glRotated(-PO[iP][14], 1.0, 0.0, 0.0 );
				// rotate axil tilt of the planet's texture
				glRotated(PO[iP][22], 0.0, 0.0, 1.0);
				// rotate the planet by time h/m/s, this reflects day/night from location
				set_material_color(mat_ambient, 0.0, 0.0, 0.0, GL_AMBIENT);
				glEnable(GL_LIGHT0);
				glDisable(GL_BLEND);
				set_light_texture_off(false);
				glCallList(list[iP][0]); 
				// draw the planet
				if (0!=PO[iP][13]) scn_moons(iP, PO[iP][13]);
				// draw the moons
				glDisable(GL_LIGHT0);				
				// disable light
				set_light_texture_off(true);
				if (iP==5) 
				{
					rot	= rot + 0.05f;
					glRotatef(rot, 0, 0, 1);	
					glCallList(list[6][9]);	
				}
				// draw it's ring if it has, so far only for Saturn, 
				// comming up next Uranus and Jupiter
			}
			glPopMatrix();
			//back to SCENE 1
			if (bool_options[4])
			// write the description for the planet if option is activated
			{
				set_2D_for_draw(0.0L, 1);
				// set screen to 2D
				fnt_PrintText_lt_off_color(true, listBase[0], false, false, wwidth - 300, wheight/2-100,  true, PO[iP][16] /* the color */, planet[iP].description );
				// write informations about the planet			
				fnt_PrintText_lt_off_color(true, listBase[0], false, false, wwidth - 300, ynewline + 15,  true, 255255255 /* the color */, "[ press space key to show/hide this description ]");
				unset_2D_for_draw();
			}
		}
SKIPP_ALL:
		glPopMatrix();
	}
	// draw atmosphere for the closest planet 
	if ((0 != list[near_planet - 1][1])||(150 >= camera_planet_distance))
	// if it planet has atmosphere then draw it
	{
		glEnable(GL_LIGHT0);
		set_light_texture_off(false);		
		glPushMatrix();
		glTranslated(PO[near_planet - 1][18], PO[near_planet - 1][19], PO[near_planet - 1][20]);
		// translate from 0 to the location of planet
		glRotated(-PO[near_planet - 1][14], 1.0, 0.0, 0.0 );
		// rotate with axial tilt of planet
		atm_rot = atm_rot + 0.007;
		glRotatef(atm_rot, 0.0, 0.0, 1.0);
		// rotate somehow the texture
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_COLOR, GL_ONE_MINUS_SRC_COLOR);
		// enable transparency for the atmosphere	
		set_material_color(mat_specular, 0.3, 0.3, 0.3, GL_SPECULAR);
		glCallList(list[near_planet - 1][1]);
		set_material_color(mat_specular, 0.0, 0.0, 0.0, GL_SPECULAR);
		glDisable(GL_BLEND);
		// leave the transparent func the way it was
		glBlendFunc(GL_SRC_ALPHA, GL_DST_ALPHA);
		// draw the texture sphere for atmosphere
		glPopMatrix();
		glDisable(GL_LIGHT0);
		set_light_texture_off(true);
	}
	camera_changed = false;
}

void load_rings()
{
	// load Saturn rings
	list[0][8] = JPEG_Texture("maps/saturnrings.JPG");
	list[6][9] = scn_ring(0.7, 2.8);
	// load Uranus rings
//	list[0][7] = JPEG_Texture("maps/uranusrings.JPG");
//	list[6][8] = scn_ring(0, 0);
	// don't have the map yet
	fnt_PrintText_lt_off_color(true, listBase[0], true, true, -0.4, -0.04,  true, 128128128 /* the color */, "Loading GUI textures");			
}

GLuint scn_ring(GLfloat inner_radius, GLfloat outer_radius)
{
	GLuint ring_list = glGenLists(1);
	// generate list for rings
	glNewList(ring_list, GL_COMPILE);
	{
		set_material_color(mat_ambient, 1.0, 1.0, 1.0, GL_AMBIENT);
		set_material_color(mat_emission, 0.65, 0.62, 0.5, GL_EMISSION);
		glEnable(GL_BLEND);					// enable blending
		glEnable(GL_TEXTURE_2D);
		glBlendFunc(GL_SRC_COLOR, GL_ONE_MINUS_SRC_COLOR);
		glDisable(GL_CULL_FACE);			// enable drawing on both sides
		glBindTexture(GL_TEXTURE_2D, list[0][8]);	
		gluDisk(qsphere, inner_radius,  outer_radius, 24, 24);	
		glEnable(GL_CULL_FACE);				// disable drawing on both sides
		glDisable(GL_BLEND);
		// leave the transparent func the way it was
		glBlendFunc(GL_SRC_ALPHA, GL_DST_ALPHA);
		set_material_color(mat_emission, 0.0, 0.0, 0.0, GL_EMISSION);
		glDisable(GL_TEXTURE_2D);
	}
	glEndList();
	return ring_list;
}

void scn_moons(unsigned short iP, unsigned short iS)
{
	double x, y, z; 

	date.Set(systemtime.wYear, systemtime.wMonth, 
		 systemtime.wDay, systemtime.wHour, systemtime.wMinute, mili, true);
	Julian = date.Julian();

	for(unsigned short i = 0; i < iS; i++)
	{
		glPushMatrix();
		// saving SCENE 1, draw in SCENE iS
		
		switch (iP)
		{
			case 2 : c_Earth_Moon(&x, &y, &z);  break;
			case 4 : c_Galilean_details(&x, &y, &z, i); break;
			case 5 : c_Saturn_moons(&x, &y, &z, i); break;
			default :
			{
				double dh, moon_rot;
				dh = (time(NULL) - load_time + systemtime.wMilliseconds / 1000.0L)/60/60 ;
				if (dh >= 23.93*364.25) { dh = dh - 23.93 * 364.25; }
				moon_rot = 360 * MO[planet[iP].moonIDs[i]][3] * dh / 23.93 / 364.25;
	
				glRotated(moon_rot, 0.0f, 0.0f, 1.0f);
				// rotate the moon around it's parent 
				dh = MO[planet[iP].moonIDs[i]][2]/100000;
				glTranslated(dh, 0, 0); 
				// translate the moon
				goto SKIPP_TRANSLATE;
			}
		}
		glTranslated(x, z,  y); 
		// translate the moon
SKIPP_TRANSLATE:
		
		fnt_PrintText_lt_off_color(true, listBase[0], true, true, 0.0f, MO[planet[iP].moonIDs[i]][0] + 0.15, true, 128128128, "%s" /* %.6f */, moon[planet[iP].moonIDs[i]].name /*, dist*/);
		// write the moon's name

		glCallList(list[iP][i + 2]);
		// i + 2 cuz in 0 is the mother planet and in 1 is it's atmosphere
		// so the moons go from 2 3 4 to 9, this makes a max of 8 moons/planet
		// draw the moon
		glPopMatrix();
		// going back again to SCENE 1
	}
}

void load_space()
{
	pixel_list = glGenLists(1);
	// generate list for space, in fact this space are many pixels 
	// set to a random x y z on a sphere with 75000 radius
	glNewList(pixel_list, GL_COMPILE);
	glPushMatrix();
	GLdouble x = 0, y = 0, r;
	for (GLfloat p=0; p<60; p += 0.1f)
	{
		if (zs == 360) zs = 0;
		zs = zs + 0.3;
		glRotatef(zs, 1, 0, 0);	
		r = (((double)rand()/((double)(RAND_MAX)+(double)(0.1))));
		// select a random gray color
		r = r + 0.3f;
		glColor3f(r, r, r);		
		r = (((double)rand()/((double)(RAND_MAX)+(double)(1))));
		r = r*(2*PI);
		x = 74000 * sin(r);
		y = 74000 * cos(r);
		glBegin(GL_POINTS);
		glVertex3f(x, y, 0);
		glEnd();
	}
	glPopMatrix();
	glEndList();
	fnt_PrintText_lt_off_color(true, listBase[0], true, true, -0.4, -0.04,  true, 128128128 /* the color */, "Killing splash screen");			
}
